/*
	File:			Timer.cc

	Function:		See header file

	Author(s):		Andrew Willmott

	Copyright:		(c) 1995-2000, Andrew Willmott

	Notes:			Any other useful timing calls?
*/


#include "cl/Timer.h"
#include <stdio.h>
#include <math.h>

Void Timer::StartTimer()
{
    startTime = CurrentTime();
	lapTime = startTime;
}

Float Timer::GetTimer()
{
    return(CurrentTime() - startTime);
}

Float Timer::DeltaTime()
{     
	Float oldLapTime = lapTime;
	
    lapTime = CurrentTime();
	
    return(lapTime - oldLapTime);
}

Void Timer::StopTimer()	/* Stops timer */
{
	stopTime = CurrentTime();
}

Void Timer::ContTimer()	/* Restarts timer */
{
	Float temp;

	temp = CurrentTime();
	temp -= stopTime;
	startTime += temp;
	lapTime += temp;
}

/* --- Unix Time ----------------------------------------------------- */


#ifdef UNIX_TIME

/* Use the times() call */

#include <unistd.h>
#include <sys/times.h>
#include <sys/time.h>
#include <errno.h>
#include "cl/String.h"

Float ProgramTimer::CurrentTime()
{
    struct tms	tb;
	Float		result;
	
    times(&tb);
    result = (Float)(tb.tms_utime) / CLK_TCK;
    if (addSystem)
    	result += (Float)(tb.tms_stime) / CLK_TCK;

	return(result);
}

Float WallClockTimer::CurrentTime()
{
    struct	timeval tv;
	Float	result;
	
    if (gettimeofday(&tv, 0))
    	_Error(String().Printf("gettimeofday call failed: %d", errno));

	result = (tv.tv_sec & 0x0FFFF) + tv.tv_usec * 1e-6;

	return(result);
}

#endif


/* --- Ansi Time ------------------------------------------------------ */


#ifdef ANSI_TIME				/* Use clock() */

#include <time.h>

Float WallClockTimer::CurrentTime()
{
	return(clock() / ((Float) CLOCKS_PER_SEC));
}

#endif


/* --- Sgi Time ------------------------------------------------------ */


#ifdef SGI_TIME

// uses SGI multimedia routines.

#include <dmedia/dmedia.h>

typedef unsigned long long UST;

Float WallClockTimer::CurrentTime()
{
	UST temp;
	
	dmGetUST(&temp);
	
	return(((Float) temp) / 1e9);
}

#endif


// --- Use rusage call ---------------------------------------------------------


#ifdef RUSAGE_TIME

#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>

Float ProgramTimer::CurrentTime()
{
	struct rusage	usage;
	Float			result;
	
	getrusage(RUSAGE_SELF, &usage);

	result = (usage.ru_utime.tv_sec & 0x0FFFF) + usage.ru_utime.tv_usec * 1e-6;
	if (addSystem)
		result += (usage.ru_stime.tv_sec & 0x0FFFF) + usage.ru_stime.tv_usec * 1e-6;

	return(result);
}

#endif
