/*
    File:       Clipper.h

	Function:   Clips homogeneous coordinates to the display frustrum
				(x, y, z) E [-1, 1] x [-1, 1] x [0, 1]

				The standard display pipeline is transform, clip, project, 
				2D draw. The Clipper class should be passed post-transform
				homogeneous coordinates. It will in turn call ViewPoint()
				with clipped coordinates. You should override ViewPoint()
				to perform the homogeneous divide, map to display coords,
				and do the drawing.

    Author:     Andrew Willmott

    Copyright:  (c) 1998-2000, Andrew Willmott
*/


#ifndef __Clipper__
#define __Clipper__

#include "gcl/Geometry.h"

typedef GCLReal BdryCoords[6];

class Clipper
{
public:
	Clipper();
	Void			ClipPoint(const HPoint &p, Bool draw);
					///< draw == false: move to p; draw == true: draw to p.
	virtual Void	ViewPoint(const HPoint &p, Bool draw) = 0;
					///< override to do actual drawing.
protected:
	HPoint			lastP;
	Int				lastOutcode;
	BdryCoords		lastBC;
};

#endif
