/*
	File:		Draw.cc

	Function:	

	Author:		Andrew Willmott

	Notes:		
*/

#include "gcl/Draw.h"
#include "gcl/VecUtil.h"
#include "gcl/Scene.h"

Int scRenderSteps = 10;

Void DrawCircle(Renderer &r, const Point &p, const Vector &n, GCLReal rad)
{
	const GCLReal	slice = 2 * vl_pi / GCLReal(scRenderSteps);
	GCLReal			theta;
	Vector			a0, a1;
	Int				i;
	
	a0 = norm(FindOrthoVector(n)) * rad;
	a1 = cross(n, a0);
	theta = 0.0;
	
	r.Begin(scPrimitive::sRenderStyle);
	
	for (i = 0; i < scRenderSteps; i++)
	{
		r.P(p + a0 * cos(theta) + a1 * sin(theta));
		theta += slice;
	}
	
	r.End();
}

Void MakeBoxCorners(const Point &min, const Point &max, Point corners[8])
/*! Creates corners as follows: 
	\verbatim

      3------2
	 /|     /|
	7------6 |
	| 0----|-1
	|/     |/
	4 ---- 5


	  y
	  |	
	  +--x
	 /
	z
	
	\endverbatim
*/
{
	corners[0] = Point(min[0], min[1], min[2]);
	corners[1] = Point(max[0], min[1], min[2]);
	corners[2] = Point(max[0], max[1], min[2]);
	corners[3] = Point(min[0], max[1], min[2]);

	corners[4] = Point(min[0], min[1], max[2]);
	corners[5] = Point(max[0], min[1], max[2]);
	corners[6] = Point(max[0], max[1], max[2]);
	corners[7] = Point(min[0], max[1], max[2]);
}

Void PaintBox(Renderer &r, const Point &min, const Point &max)
{
	Point	corners[8];
	
	MakeBoxCorners(min, max, corners);
	PaintBox(r, corners);
}

Void FrameBox(Renderer &r, const Point &min, const Point &max)
{
	Point	corners[8];
	
	MakeBoxCorners(min, max, corners);
	FrameBox(r, corners);
}

Void PaintRect(Renderer &r, const Coord &min, const Coord &max)
{
	r.Begin(scPrimitive::sRenderStyle)
		.P(min).P(Coord(max[0], min[1]))
		.P(max).P(Coord(min[0], max[1]))
		.P(min)
		.End();
}

Void FrameRect(Renderer &r, const Coord &min, const Coord &max)
{
	r.Begin(renLineLoop)
		.P(min).P(Coord(max[0], min[1]))
		.P(max).P(Coord(min[0], max[1]))
		.P(min)
		.End();
}

Void PaintBox(Renderer &r, Point c[8])
{
	r.N(-Vector(vl_z));
	r.Begin(scPrimitive::sRenderStyle).P(c[3]).P(c[2]).P(c[1]).P(c[0]).End();
	r.N(Vector(vl_z));
	r.Begin(scPrimitive::sRenderStyle).P(c[4]).P(c[5]).P(c[6]).P(c[7]).End();

	r.N(-Vector(vl_y));
	r.Begin(scPrimitive::sRenderStyle).P(c[0]).P(c[1]).P(c[5]).P(c[4]).End();
	r.N(Vector(vl_x));
	r.Begin(scPrimitive::sRenderStyle).P(c[1]).P(c[2]).P(c[6]).P(c[5]).End();
	r.N(Vector(vl_y));
	r.Begin(scPrimitive::sRenderStyle).P(c[2]).P(c[3]).P(c[7]).P(c[6]).End();
	r.N(-Vector(vl_x));
	r.Begin(scPrimitive::sRenderStyle).P(c[3]).P(c[0]).P(c[4]).P(c[7]).End();
}

Void FrameBox(Renderer &r, Point c[8])
{
	r.Begin(renLineLoop).P(c[0]).P(c[1]).P(c[2]).P(c[3]).End();
	r.Begin(renLineLoop).P(c[4]).P(c[5]).P(c[6]).P(c[7]).End();

	r.Begin(renLines)
		.P(c[0]).P(c[4])
		.P(c[1]).P(c[5])
		.P(c[2]).P(c[6])
		.P(c[3]).P(c[7])
		.End();
}

