/*
	File:		Cluster.h

	Function:	Implements hierarchical radiosity clusters

	Author:		Andrew Willmott

    Copyright:  (c) 1997-2000, Andrew Willmott
*/

#ifndef __Cluster__
#define __Cluster__

#include "HRElem.h"


class Cluster : public HRElem
{
public:
				Cluster();
	virtual		~Cluster();

	Void		SetElems(HRElemList *elemIn);
	Cluster		*CreateHierarchy();

	Void		PushElems();
	Void		CreateBounds();

	Void		DrawNodeElem(Renderer &r);
	Void		DrawLeafElem(Renderer &r);
	Void		DrawPatches(Renderer &r);

	Void		Print(ostream &s);

	// radiosity routines.	
	Void		Reset();
	Bool		IsLeaf();
	Void		ApplyToChildren(Void (HRElem::*method)(Void*), Void *ref = 0);

	Void		EltUpdateBounds(Point &, Point &)
				{ return; };
	GCLReal		EltArea()	
				{ return(clusterArea); };
	GCLReal		EltProjArea(const Vector &n);
	GCLReal		EltMaxProjArea(const Vector &n);
	Point		EltCentre() 
				{ return(centre); };
	Colour		EltE()		
				{ return(E); };
	Colour		EltRho()	
				{ return(maxRho); };
	Void		EltSampleTransport(Int numSamples, Point p[], Vector n[],
						Matd &coeffs);
	GCLReal		EltCalcTransport(HRElem *from, Matd &coeffs);

	Void		AddIrradiance(const Colour &E, const Vector &m);
	Colour		GetPower(const Vector &m);
	Void		EltGetSamples(Int numSamples, Point pts[]);

	Void		DistributeColours();
	Void		DistributeColoursBest(ShadeInfo &shadeInfo);

	GCLReal		Error();
	Void		Add();
	Void		Push();
	Void		Pull();
	Void		ClearB();
	Void		ClearR();
	Void		CalcLeafRadiosity();
	Void		InitRad();

	Void		MakeChildLinks(HRElem *other, HRLink *link,
							   Int which, Int levels);
	Void		EltSetVisPoints(HRElem *to, Point p[]);

	// Misc
	Int			NumCoeffs() { return(1); }
	Colour		*B_Coeffs() { return(&B); };
	Colour		*R_Coeffs() { return(&R); };

	// Fields
	Point		min, max;		// bounds
	Point		centre;
	Int			level;
	HRElemList	elems;			// non-cluster elements held by this cluster
	Cluster		*child[8];		// child clusters

	GCLReal	   	clusterVol;		// volume of the cluster
	GCLReal	   	clusterArea;	// surface area contained by cluster
	Colour	   	maxRho;			// max reflectivity in cluster
	Colour	   	E;				// emittance -- currently unused.
	Colour	   	B;
	Colour	   	R;
	static Colour lastB;
};

#endif
