/*
	File:			HierRad.h

	Function:		Provides a class for lighting a scene using hierarchical
					(wavelet) radiosity.
					
	Author(s):		Andrew Willmott

	Copyright:		(c) 1997-2000, Andrew Willmott
 */

#ifndef __HierRad__
#define __HierRad__

#include "RadMethod.h"
#include "HRElem.h"
#include "Cluster.h"

class HierRad : public RadMethod
{
public:
	HierRad();
	~HierRad();
	
	virtual HRLink
				*CreateLink();
				// create a new elem->elem link

	Void		SetScene(scScenePtr scene);
	Void		ResetOptions();
	RadElem		*NewMesh();
	Int			Stage(Int stage);
	Void		Draw(Renderer &r);
	Void 		DrawMatrix(Renderer &r);
	Void		DumpStats();
	Void		ColourVertices();
	PatchList	*GetElements();

	Void		PrintCmds(ostream &s) const;
	Bool		ParseCmd(StrConst str, istream &s);

	// Render methods
	Bool		Render();				// override

	// Linking methods
	Bool		CreateClusterLink();
				// for clustering
	Bool		CreateInitialPatchLinks();
				// for normal wavelet radiosity.

	// Solution methods
	Bool		ScheduledSolve();
	Bool		MultiGridSolve();
	Bool		TwoStageSolve();

	// Extras
	HRElem		*ClosestHRIntersection(Point &start, Vector &direction, 
									 Point &hitPoint);

	// Fields
	HRElemList	clusterableElems;	///< Clusterable HR elements
	Cluster		*cluster;			///< Cluster
	HRElemPtr	*trees;				///< list of trees in the scene
	Int			numTrees;
	HRElemPtr	clusTree;			///< points to top-level scene cluster
	PatchList	*leaves;

	Int			iterations;
	Int			poly;
	GCLReal		error;
	Int			linkPatch;
	Int			linksCreated;
	Int			linksCreatedLast;


	static Void	*New() { return(new HierRad); };
};

#endif
