/*
	File:			RadControl.cc

	Function:		See header file

	Author(s):		Andrew Willmott

	Copyright:		Copyright (c) 2000, Andrew Willmott

	Notes:			

*/

#include "RadControl.h"


Int	radVersionMajor = 2;
Int	radVersionMinor = 1;
Int	radVersionFix = 1;

TempString RadGetVersion()
{
	TempString rv;

	rv.Printf("%d.%d.%d",
			radVersionMajor, radVersionMinor, radVersionFix);	

	return(rv);
}

StrConst RadGetCopyright()
{
	static StrConst rc =
			"(c) Andrew Willmott <ajw+rad@cs.cmu.edu> 2000";

	return(rc);
}

Reflectance kRadRGBToLum(1.0 / 3.0, 1.0 / 3.0, 1.0 / 3.0);

RadControl *gRadControl = 0;


// --- Default radiosity options ----------------------------------------------


RadControl::RadControl() : 

	// solution technique
	method(kMatrix),
	basis(kHaar),

	// method parameters
	patchSubdivs(2.0),
	eltSubdivs(2.0),
	alpha(1.0), 
	error(0.01), 

	kAError(1e-6),
	kFError(0.01),

	solver(sv_scheduled),
	useBF(true),
	cluster(true),
	gridOn(true),
	textureRefl(false),
	useConjGrad(false),
	ambient(false),
	refAllLinks(true),
	refineLevels(4),
	maxShots(-1),
	schedIterations(3),

	// visibility
	visibility(vis_4x4),
	jitterRot(true),
	dFError(0.05),
	visError(0.1),
	quadLevel(1),
	forceVisReuseArea(0.2),
	sampleFactor(20),
	visInQuad(false),
	favourReceiver(1.0),

	// meshing
	mesh(mesh_std),
	fixMesh(false),
	connectMesh(true),
	noGridMesh(false),

	// MR complexity
	rtComplexity(1.0),
	meshComplexity(1.0),

	// simulation	
	stop(false),
	step(false),
	pause(false),

	// display options
	graded(true),
	anchor(true),
	gouraud(false),
	texture(false),
	wire(false),
	redWire(false),
   	shotDisplay(0),
	patchView(0),
	pvData(0),
	funcView(0),
	animate(0),
	showRays(false),
	showLinks(false),
	showFCs(false),
	choke(0),
	drawMatrix(true),
	outlineClusters(false),
	outlineVisGrid(false),

	finalPass(false),
	bestPass(false),
	bestVisPass(false),
	bestLevels(0),

	// render objects
	radObject(0),
	stage(0),

	// render stats	
	dumpTree(false),
	dumpScenes(false),
	updateScene(false),
	sliceTime(1e10),	// dump stats only after termination.
	limitTime(1e10),	// don't place any time limit on termination
	numPolys(0),
	rays(0),
	mem(0.0),
	outFile("out"),

	// extra for debugging & hacking in new stuff.
	o1(false), o2(false), o3(false), o4(false), o5(false), o6(false)
{
}

