/*
	File:		RadPane.cc

	Function:	Display pane for radiosity mesh

	Author:		Andrew Willmott

	Notes:		
*/

#include "RadPane.h"
#include "RadMethod.h"
#include "HierRad.h"
#include "HRMesh.h"

#include "gcl/Scene.h"
#include "gcl/Camera.h"

Void FormsScenePane::SetParent(Form *newParent)
{
	parent = newParent;
}


// --- RadScenePane class -----------------------------------------------------


RadScenePane::RadScenePane(Bool doubleBuf) : FormsScenePane(doubleBuf)
{
	selPatch[0] = 0;
	selPatch[1] = 0;
	selElem[0] = 0;
	selElem[1] = 0;
	enabled = false;
}

Void RadScenePane::SetScene(scScenePtr scene)
{
	Int		i;

	enabled = false;
	selPatch[0] = 0;
	selPatch[1] = 0;
	ScenePane::SetScene(scene);
	mesh = gRadControl->radObject;


	enabled = true;
	Redraw();
}

Void RadScenePane::Redraw()
{
	if (enabled)
		ScenePane::Redraw();
}

Void RadScenePane::StartDrag(Coord c, Int modifiers)
{
	if (modifiers & wmButton3)
		inverseTransform = inv(itsCamera->ProjMatrix() * 
							   itsCamera->ModelMatrix());
	ScenePane::StartDrag(c, modifiers);
}

Void RadScenePane::EndDrag(Coord c, Int modifiers)
{
	ScenePane::EndDrag(c, modifiers);


	if (parent)
		parent->Event(msgPick);
}

Void RadScenePane::Drag(Coord c, Int modifiers)
{
	offset = c - dragStart;

	if (modifiersStart & wmButton3)
	{
		if (modifiersStart & wmControl)
			ComplexityDrag();
		else
			SelectDrag();
	}
	else
		ScenePane::Drag(c, modifiers);
}

Void RadScenePane::SelectDrag()
{
	if (!mesh)
		return;

	RadElem		 	*pickedElem;
	HRElem			*pickedHRElem;
	Point 			p, from;
	Vector			dir;
	Coord 			coord, start;
	Int				i, which;
	Int				levelsUp = (Int) (8.0 * sqrt(len(offset)));

	// selecting light source or receiver
	which = 0;
	if (modifiersStart & wmShift)
		which++;

	// de-highlight previous selection
	if (selPatch[which])
	{
		if (which == 0)
			gRadControl->pvData = 0;	
		selPatch[which]->SetHighlight(0);
		selPatch[which] = 0;
	}
	if (selElem[which])
	{
		if (which == 0)
			gRadControl->pvData = 0;	
		selElem[which]->SetHighlight(0);
		selElem[which] = 0;
	}

	if (modifiersStart & wmAlt)
		return;

	start[0] = 2.0 * dragStart[0] - 1.0;
	start[1] = 2.0 * dragStart[1] - 1.0;

	FindWorldRay(start, inverseTransform, from, dir);

	if (gRadControl->method != kHierarchical && 
		(pickedElem = mesh->ClosestIntersection(from, dir, p)))
	{
		coord = pickedElem->FindCoord(p);

		if (selPatch[which])
			selPatch[which]->SetHighlight(0);
		else if (selElem[which])
			selElem[which]->SetHighlight(0);
			
		if (gRadControl->method == kProgSubstruct)
		{
			selPatch[which] = pickedElem->FindContainer(coord);
			
			// if this is a light, select a patch rather than an element.
			if (which == 1)
				levelsUp = 1000;
			
			for (i = 0; i < levelsUp && ((HierElem*) selPatch[which])->parent;
				 i++)
					selPatch[which] = ((HierElem*) selPatch[which])->parent;

			if (i < levelsUp)
				if (which == 0)
					// jump to the container element 
					selPatch[which] = pickedElem;
		}				
		else
		{
			if (levelsUp < 3)
				selPatch[which] = pickedElem->FindContainer(coord);
			else
				selPatch[which] = pickedElem;
		}

		if (selPatch[which])
			selPatch[which]->SetHighlight(2 + which);
		else
		{
			cerr << "Bad coord: " << coord << endl;
			_Error("(SelectDrag) Bad Coordinate");
		}
		gRadControl->pvData = 0;
	}
	else if (gRadControl->method == kHierarchical && 
		(pickedHRElem = ((HierRad*) mesh)->ClosestHRIntersection(from, dir, p)))

	{
		HierElem	*patchElem = 0;

		if (selPatch[which])
			selPatch[which]->SetHighlight(0);
		else if (selElem[which])
			selElem[which]->SetHighlight(0);

		i = 0;

		if (pickedHRElem->IsPatch())
		{
			patchElem = dynamic_cast<HRMeshElem*>(pickedHRElem);
			coord = patchElem->FindCoord(p);
			patchElem = dynamic_cast<HierElem*>(patchElem->FindContainer(coord));

			for (; i < levelsUp && patchElem->parent; i++)
					patchElem = patchElem->parent;
			
			pickedHRElem = dynamic_cast<HRMeshElem*>(patchElem);
		}

		// continue up HR hierarchy
		for (; i < levelsUp && pickedHRElem->eltParent; i++)
			pickedHRElem = pickedHRElem->eltParent;

		if (pickedHRElem->IsPatch())
		{
			selPatch[which] = patchElem;
			selElem[which] = pickedHRElem;
		}	
		else
		{
			selPatch[which] = 0;
			selElem[which] = pickedHRElem;
		}
		pickedHRElem->SetHighlight(2 + which);
		gRadControl->pvData = pickedHRElem;
	}

	Redraw();
}
