/*
	File:			RadScene.h

	Function:		Defines various useful GCL scene primitives
					
	Author(s):		Andrew Willmott

	Copyright:		(c) 1997-2000, Andrew Willmott
*/

#ifndef __RadScene__
#define __RadScene__

#include "RadMesh.h"
#include "gcl/SceneObjects.h"

class scRadMesh : public scPrimitive
/** Defines a 'mesh' GCL object that encapsulates a radiosity method.
	This lets us display, save, and reload the mesh using the normal
	GCL mechanisms. To read a scene file that contains a radiosity mesh,
	you must add this object to the scene library first via a 
		slAddToLibrary("mesh", new scRadMesh); XXX
	Note: this is suffering from code rot a little...
*/
{
public:
	scRadMesh() : scPrimitive(pExtension), rad(0) {};

	Object		 	*Clone() const { return new scRadMesh(SELF); };

	Void 			Draw(Renderer &r, SLContext *context);

	//	Parsing

	Void			Print(ostream &s) const;
	Void			Parse(istream &s);
			
 	// Radiosity methods
	
//	Void 			Compare(scPrimitive *to, GCLReal edgeLen, 
//						CompareStats &stats);

	Void			SetMethod(RadMethod *rm) { rad = rm; };

protected:
	RadMethod		*rad;
};

class scMatrixDisplay : public scPrimitive
/** Draw a representation of the radiosity matrix in the x-y plane.
*/
{
public:
	scMatrixDisplay(Void **method) : scPrimitive(pExtension), method(method) {};
	scMatrixDisplay(const scMatrixDisplay &md) : scPrimitive(md), method(md.method) {};

	Object		 			*Clone() const { return new scMatrixDisplay(SELF); };

	Void					Draw(Renderer &r, SLContext *context);
	StrConst 				Label() const;

	Void					**method;
};

#endif
